package com.gic.haoban.manage.service.service.impl;

import java.util.*;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.enums.BindTypeEnum;
import com.gic.haoban.manage.api.enums.ChannelCodeEnum;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanMemberClerkChatConfigMapper;
import com.gic.haoban.manage.service.entity.MemberClerkChatConfig;
import com.gic.haoban.manage.service.entity.TabHaobanClerkMainStoreRelated;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkBindLog;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkRelation;
import com.gic.haoban.manage.service.service.ClerkMainStoreRelatedService;
import com.gic.haoban.manage.service.service.ExternalClerkRelatedService;
import com.gic.haoban.manage.service.service.StaffClerkBindLogService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.wechat.api.dto.qywx.QywxExternalcontactDTO;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import javafx.scene.control.Tab;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.StaffClerkBindLogDetailDTO;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStaffClerkRelationMapper;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;

@Service
public class StaffClerkRelationServiceImpl implements StaffClerkRelationService {

    private static final Logger logger = LoggerFactory.getLogger(StaffClerkRelationServiceImpl.class);

    @Autowired
    private TabHaobanStaffClerkRelationMapper mapper;

    @Autowired
    private StaffClerkBindLogService staffClerkBindLogService;

    @Autowired
    private ExternalClerkRelatedService externalClerkRelatedService;

    @Autowired
    private StaffService staffService;

    @Autowired
    private TabHaobanStaffClerkRelationMapper tabHaobanStaffClerkRelationMapper;

    @Autowired
    private ClerkMainStoreRelatedService clerkMainStoreRelatedService;

    @Autowired
    private TabHaobanMemberClerkChatConfigMapper memberClerkChatConfigMapper;

    @Override
    public List<StaffClerkRelationDTO> listBindCode(String enterpriseId, Set<String> clerkCodeList) {
        return EntityUtil.changeEntityListByJSON(StaffClerkRelationDTO.class, mapper.listBindCode(enterpriseId, clerkCodeList));
    }

    @Override
    public List<StaffClerkRelationDTO> listBindCodeByStaffId(List<String> enterpriseIdList, String staffId) {
        return EntityUtil.changeEntityListByJSON(StaffClerkRelationDTO.class, mapper.listBindCodeByStaffId(enterpriseIdList, staffId));
    }

    @Override
    public boolean delBind(String clerkId, String optStaffId, int chanelCode) {
        TabHaobanStaffClerkRelation clerkRelation = mapper.getByClerkId(clerkId);
        if (null == clerkRelation) {
            logger.info("已经不存在：{},{}", clerkId, optStaffId);
            return true;
        }
        TabHaobanStaffClerkRelation staffClerkRelation = tabHaobanStaffClerkRelationMapper.getOneByClerkId(clerkId);
        int i = mapper.changeStatusByClerkId(clerkId, 0);
        delSetMainStore(staffClerkRelation);
        boolean b = i >= 0;
        if (b) {
            //推入日志
            staffClerkBindLogService.pushToMq(clerkRelation.getStaffId(), optStaffId, BindTypeEnum.UNBIND.getVal(), chanelCode, clerkRelation.getStaffClerkRelationId());
        }
        return b;
    }

    private void delSetMainStore(TabHaobanStaffClerkRelation staffClerkRelation) {
        if (staffClerkRelation == null) {
            logger.info("staffClerkRelation为空");
            return;
        }
        String staffId = staffClerkRelation.getStaffId();
        String wxEnterpriseId = staffClerkRelation.getWxEnterpriseId();
        TabHaobanClerkMainStoreRelated r = clerkMainStoreRelatedService.selectByWxEnterpriseIdAndStoreId(staffId, wxEnterpriseId);
        if(r != null){
            r.setStatusFlag(0);
            clerkMainStoreRelatedService.update(r);
        }
        externalClerkRelatedService.delByClerkIdAndStaffId(staffClerkRelation.getClerkId(), staffId);
        List<StaffClerkRelationDTO> list = this.lisByStaffId(wxEnterpriseId, staffId);
        if (list != null && list.size() >= 1) {
            clerkMainStoreRelatedService.setMainStore(staffId, list.get(list.size() - 1).getStoreId(), wxEnterpriseId);
        }
    }

    private void setMainStore(StaffClerkRelationDTO staffClerkRelation) {
        if (staffClerkRelation == null) {
            logger.info("staffClerkRelation为空");
            return;
        }
        String staffId = staffClerkRelation.getStaffId();
        String wxEnterpriseId = staffClerkRelation.getWxEnterpriseId();
        List<StaffClerkRelationDTO> list = this.lisByStaffId(wxEnterpriseId, staffId);
        if (list != null && list.size() == 1) {
            clerkMainStoreRelatedService.setMainStore(staffId, staffClerkRelation.getStoreId(), wxEnterpriseId);
        }
    }

    @Override
    public String bind(StaffClerkRelationDTO dto, String optStaffId, int chanelCode) {
        dto.setStaffClerkRelationId(ToolUtil.randomUUID());
        dto.setStatusFlag(1);
        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());

        int insert = mapper.insert(EntityUtil.changeEntityByJSON(TabHaobanStaffClerkRelation.class, dto));
        setMainStore(dto);
        String s = insert > 0 ? dto.getStaffClerkRelationId() : null;
        if (null != s) {
            //推入日志
            staffClerkBindLogService.pushToMq(dto.getStaffId(), optStaffId, BindTypeEnum.BIND.getVal(), chanelCode, dto.getStaffClerkRelationId());
        }
        return s;
    }

    @Override
    public boolean updateByClerkId(StaffClerkRelationDTO dto) {
        if (StringUtils.isAnyBlank(dto.getClerkCode(), dto.getClerkId(), dto.getStoreId())) {
            logger.info("操作不对：{}", JSONObject.toJSONString(dto));
            return false;
        }
        int i = mapper.updateByClerkId(EntityUtil.changeEntityByJSON(TabHaobanStaffClerkRelation.class, dto));
        return i >= 0;
    }


    @Override
    public StaffClerkRelationDTO getBindByClerkId(String clerkId, String wxEnterpriseId) {
        TabHaobanStaffClerkRelation relation = mapper.getBindByClerkId(clerkId, wxEnterpriseId);
        return EntityUtil.changeEntityByJSON(StaffClerkRelationDTO.class, relation);
    }

    @Override
    public StaffClerkRelationDTO getByCodeAndEnterpriseId(String clerkCode, String enterpriseId) {
        return EntityUtil.changeEntityByJSON(StaffClerkRelationDTO.class, mapper.getByCodeAndEnterpriseId(clerkCode, enterpriseId));
    }

    @Override
    public String insert(StaffClerkRelationDTO staffClerkRelation) {
        String uuid = StringUtil.randomUUID();
        staffClerkRelation.setStaffClerkRelationId(uuid);
        staffClerkRelation.setCreateTime(new Date());
        staffClerkRelation.setUpdateTime(new Date());
        staffClerkRelation.setStatusFlag(1);
        TabHaobanStaffClerkRelation relation = EntityUtil.changeEntityByJSON(TabHaobanStaffClerkRelation.class, staffClerkRelation);
        mapper.insert(relation);
        return uuid;
    }

    @Override
    public int cleanStaffClerk(String wxEnterpriseId, List<String> delStaffIds) {
        delStaffIds.forEach(staffId -> {
            List<StaffClerkRelationDTO> relationDTOS = listBindCodeByStaffId(null, staffId);
            if (CollectionUtils.isNotEmpty(relationDTOS)) {
                relationDTOS.forEach(mid -> {
                    staffClerkBindLogService.pushToMq(mid.getStaffId(), "-1", BindTypeEnum.UNBIND.getVal(), ChannelCodeEnum.WEIXIN_DEL.getCode(), mid.getStaffClerkRelationId());
                });
            }
        });
        int ret = mapper.cleanStaffDepart(wxEnterpriseId, delStaffIds);
        externalClerkRelatedService.cleanStaffRelated(wxEnterpriseId, delStaffIds);
        return ret;
    }

    @Override
    public StaffClerkRelationDTO getOneBindCodeNoStatus(String enterpriseId, String clerkCode) {
        return EntityUtil.changeEntityByJSON(StaffClerkRelationDTO.class, mapper.getOneBindCodeNoStatus(enterpriseId, clerkCode));
    }

    @Override
    public StaffClerkRelationDTO getOneBindByStoreId(String staffId, String storeId) {
        return EntityUtil.changeEntityByJSON(StaffClerkRelationDTO.class, mapper.getOneBindStoreId(staffId, storeId));
    }

    @Override
    public StaffClerkRelationDTO getByClerkId(String clerkId) {
        return EntityUtil.changeEntityByJSON(StaffClerkRelationDTO.class, mapper.getByClerkId(clerkId));
    }

    @Override
    public List<StaffClerkRelationDTO> listByClerkIds(List<String> clerkList) {
        if (clerkList == null || clerkList.isEmpty()) {
            return Collections.EMPTY_LIST;
        }
        return EntityUtil.changeEntityListByJSON(StaffClerkRelationDTO.class, mapper.listByClerkIds(clerkList));
    }

    @Override
    public List<StaffClerkRelationDTO> lisByStaffId(String wxEnterpriseId, String staffId) {
        return EntityUtil.changeEntityListByJSON(StaffClerkRelationDTO.class, mapper.lisByStaffId(wxEnterpriseId, staffId));
    }

    @Override
    public void saveChatConfig(MemberClerkChatConfig memberClerkChatConfig) {
        memberClerkChatConfig.setStatus(1);
        memberClerkChatConfig.setCreateTime(new Date());
        memberClerkChatConfig.setUpdateTime(new Date());
        memberClerkChatConfigMapper.insert(memberClerkChatConfig);
    }

    @Override
    public MemberClerkChatConfig getChatConfig(String staffId, String memberId) {
        MemberClerkChatConfig chatConfig = memberClerkChatConfigMapper.getChatByMemberIdStaffId(memberId, staffId);
        return chatConfig;
    }

	@Override
	public List<TabHaobanStaffClerkRelation> listByWxEnterpriseId(String wxEnterpriseId) {
		return mapper.listByWxEnterpriseId(wxEnterpriseId);
	}

	@Override
	public Page<StaffClerkRelationDTO> pageByWxEnterpriseId(String wxEnterpriseId, BasePageInfo pageInfo) {
		PageHelper.startPage(pageInfo);
        List<TabHaobanStaffClerkRelation> clerkBindList = mapper.pageByWxEnterpriseId(wxEnterpriseId);
        Page<StaffClerkRelationDTO> retPage = PageUtil.changePageHelperToCurrentPage(new PageInfo<>(clerkBindList), StaffClerkRelationDTO.class);
        return retPage;
	}

    @Override
    public List<StaffClerkRelationDTO> listBindByStoreId(String wxEnterpriseId, String storeId) {
        List<TabHaobanStaffClerkRelation> ret = mapper.listBindByStoreId(wxEnterpriseId, storeId);
        return EntityUtil.changeEntityListNew(StaffClerkRelationDTO.class, ret);
    }
}
