package com.gic.haoban.manage.service.service.impl;

import com.gic.api.base.commons.Page;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.StoreOrGroupInfoDTO;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStoreRangeMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStoreRelationMapper;
import com.gic.haoban.manage.service.entity.TabStoreRange;
import com.gic.haoban.manage.service.entity.TabStoreRelation;
import com.gic.haoban.manage.service.service.StoreRangeService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created 2021/4/9.
 *
 * @author hua
 */
@Service
public class StoreRangeServiceImpl implements StoreRangeService {

    @Autowired
    private TabHaobanStoreRangeMapper tabHaobanStoreRangeMapper;

    @Autowired
    private TabHaobanStoreRelationMapper tabHaobanStoreRelationMapper;


    @Override
    public boolean saveStoreRange(List<StoreOrGroupInfoDTO> relations, String enterpriseId, String wxEnterpriseId) {
        tabHaobanStoreRangeMapper.deleteByEnterpriseId(wxEnterpriseId, enterpriseId);
        List<TabStoreRange> ranges = relations.stream().map(dto -> {
            TabStoreRange tabStoreRange = new TabStoreRange();
            tabStoreRange.setEnterpriseId(enterpriseId);
            tabStoreRange.setWxEnterpriseId(wxEnterpriseId);
            tabStoreRange.setRelationId(dto.getRelationId());
            tabStoreRange.setRelationType(dto.getRelationType());
            tabStoreRange.setStoreRangeId(ToolUtil.randomUUID());
            return tabStoreRange;
        }).collect(Collectors.toList());
        tabHaobanStoreRangeMapper.insertBatch(ranges);
        return true;
    }

    @Override
    public List<StoreOrGroupInfoDTO> queryBindStoreRange(String enterpriseId, String wxEnterpriseId) {
        List<TabStoreRange> tabStoreRanges = tabHaobanStoreRangeMapper.queryByEnterpriseId(wxEnterpriseId, enterpriseId);
        if (CollectionUtils.isEmpty(tabStoreRanges)) {
            return null;
        }
        return tabStoreRanges.stream().map(tab -> {
            StoreOrGroupInfoDTO infoDTO = new StoreOrGroupInfoDTO();
            infoDTO.setRelationId(tab.getRelationId());
            infoDTO.setRelationType(tab.getRelationType());
            return infoDTO;
        }).collect(Collectors.toList());
    }

    @Override
    public List<TabStoreRange> queryAllBindRangeByEnterpriseId(String enterpriseId) {
        List<TabStoreRange> tabStoreRanges = tabHaobanStoreRangeMapper.queryByEnterpriseId(null, enterpriseId);
        if (CollectionUtils.isEmpty(tabStoreRanges)) {
            return null;
        }
        return tabStoreRanges;
    }

    @Override
    public boolean delStoreRangeByRelationId(String enterpriseId, String relationId) {
        tabHaobanStoreRangeMapper.delStoreRangeByRelationId(relationId, enterpriseId);
        return true;
    }

    @Override
    public boolean delStoreRelationByStoreId(String enterpriseId, String storeId) {
        tabHaobanStoreRelationMapper.deleteStoreRalation(enterpriseId, storeId);
        return true;
    }

    @Override
    public Page<TabStoreRelation> pageStoreRelation(String enterpriseId, String wxEnterpriseId, String search, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo);
        List<TabStoreRelation> storeRelations = tabHaobanStoreRelationMapper.searchStoreRange(wxEnterpriseId, enterpriseId, search);
        Page<TabStoreRelation> retPage = PageUtil.changePageHelperToCurrentPage(new PageInfo<>(storeRelations));
        return retPage;
    }

    @Override
    public boolean saveStoreRelations(List<TabStoreRelation> list) {
        //todo 销售线索 批量操作
        list.forEach(tab -> {
            tabHaobanStoreRelationMapper.insertSelective(tab);
        });
        return true;
    }
}
