package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreGroupDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.*;
import com.gic.haoban.manage.api.service.ClerkMainStoreRelatedApiService;
import com.gic.haoban.manage.api.service.DealSyncOperationApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.service.entity.TabHaobanClerkMainStoreRelated;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkRelation;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.*;
import com.gic.haoban.manage.service.task.BaseSyncOperation;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.service.qywx.QywxDepartmentApiService;
import com.task.allocation.api.AbstractTaskAllocationNewOperation;
import com.task.allocation.exception.TaskAllocationException;
import com.task.allocation.qo.TaskAllocationComputed;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collector;
import java.util.stream.Collectors;

/**
 * Created 2020/4/9.
 *
 * @author hua
 */
@Service
public class DealSyncOperationApiServiceImpl implements DealSyncOperationApiService {

    private static final Logger logger = LoggerFactory.getLogger(DealSyncOperationApiServiceImpl.class);

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private PreDealService preDealService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private SyncTaskService syncTaskService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private Map<String, BaseSyncOperation> operationMap;

    @Autowired
    private StaffService staffService;

    @Autowired
    private StaffDepartmentRelatedService staffDepartmentRelatedService;

    @Autowired
    private ClerkService clerkService;

    @Autowired
    private QywxDepartmentApiService qywxDepartmentApiService;

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Autowired
    private StaffClerkRelationService staffClerkRelationService;

    @Autowired
    private SecretSettingService secretSettingService;
    
    @Autowired
    private ClerkMainStoreRelatedService clerkMainStoreRelatedService;

    private static final String LOCK_KEY = "sync_wx_lock_";

    private static final String LOCK_SUCCESS_KEY = "sync_wx_success_lock_";

    private static final String FRIEND_LOCK = "haoban_fresh_wx_friend_lock_";

    @Override
    public void departmentDealMq(String params) {
        logger.info("mq处理中:{}", params);
        DealParamMqDTO dealParamMqDTO = JSONObject.parseObject(params, DealParamMqDTO.class);
        TabHaobanPreDealLog dataPre = preDealService.getByDataId(dealParamMqDTO.getTaskId()
                , dealParamMqDTO.getData(),dealParamMqDTO.getType()-1, PreDealStatusEnum.all.getVal());

        if (dataPre == null) {
            logger.info("待处理数据不存在:{}", params);
            return;
        }
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(dealParamMqDTO.getTaskId());
        //部门处理
        if (dataPre.getDataType().equals(PreDealTypeEnum.group.getVal())) {
            if (syncTask.getTaskType().equals(0)) {
                operationMap.get("groupSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);
            } else {
                operationMap.get("qywxGroupSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);
            }
            //门店处理
        } else if (dataPre.getDataType().equals(PreDealTypeEnum.store.getVal())) {
            logger.info("门店处理：{}", params);
            operationMap.get("storeSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);

            //成员处理
        } else if (dataPre.getDataType().equals(PreDealTypeEnum.clerk.getVal())) {
            logger.info("店员处理：{}", params);
            if (syncTask.getTaskType().equals(0)) {
                operationMap.get("clerkSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);
            } else {
                operationMap.get("qywxClerkSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);
            }
        }else if(dataPre.getDataType().equals(PreDealTypeEnum.friend_clerk.getVal())){
        	logger.info("好友导购处理：{}", params);
            operationMap.get("friendClerkSyncNewOperation").dealSingleByMq(dealParamMqDTO, dataPre);
        }else if(dataPre.getDataType().equals(PreDealTypeEnum.friend.getVal())){
            operationMap.get("friendSyncNewOperation").dealSingleByMq(dealParamMqDTO, dataPre);
            logger.info("好友处理：{}", params);
        	
        }else if(dataPre.getDataType().equals(PreDealTypeEnum.self_friend.getVal())){
            operationMap.get("selfFriendSyncNewOperation").dealSingleByMq(dealParamMqDTO, dataPre);
            logger.info("好友处理：{}", params);
        	
        } else if(dataPre.getDataType().equals(PreDealTypeEnum.friend_merge.getVal())){
        	operationMap.get("friendMergeSyncOperation").dealSingleByMq(dealParamMqDTO, dataPre);
        	logger.info("好友合并处理：{}", params);
        	
        }  else {
            logger.info("不是正常数据:{}", params);
        }

    }


    @Override
    public void dealDepartment(String taskId, String wxEnterpriseId
            , List<SyncSingleDealDTO> addList, List<SyncSingleDealDTO> editList) {

        logger.info("初始开始:{}", taskId);
        //处理修改
        dealEidtSync(editList);

        if (CollectionUtils.isEmpty(addList)) {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
            unlockTask(wxEnterpriseId);
            return;
        }
        logger.info("处理分组信息开始：{}", taskId);
        dealAddSync(taskId, wxEnterpriseId, addList);

        //加入队列处理
        logger.info("初始化到队列：{}", taskId);
        dealDepartmentInit(taskId, wxEnterpriseId, addList);

    }

    @Override
    public void reDealDepartment(String taskId, int dataType) {
        logger.info("开始重新同步:{}", taskId);
        TabHaobanSyncTask task = syncTaskService.getSyncTask(taskId);
        if (null == task) {
            logger.info("任务不存在!{}", taskId);
            return;
        }
        if ((!task.getStatusFlag().equals(SyncTaskStatusEnum.exception_close.getVal())) &&
                (!task.getStatusFlag().equals(SyncTaskStatusEnum.exception_compute.getVal()))) {
            logger.info("任务不在没有在异常状态!{}", taskId);
            return;
        }

        int groupErrCount = preDealService.countExcepAndPreByTaskId(taskId, PreDealTypeEnum.group.getVal());
        int storeErrCount = preDealService.countExcepAndPreByTaskId(taskId, PreDealTypeEnum.store.getVal());
        List<TabHaobanPreDealLog> list = null;
        if (dataType == -1) {
            if (groupErrCount > 0) {
                dataType = PreDealTypeEnum.group.getVal();
                list = preDealService.queryDepartRebuildDealLog(taskId);
            } else if (storeErrCount > 0) {
                dataType = PreDealTypeEnum.store.getVal();
                list = preDealService.listRebuildByTaskId(taskId, dataType);
            } else {
                dataType = PreDealTypeEnum.clerk.getVal();
                list = preDealService.listRebuildByTaskId(taskId, dataType);
            }
        }
        if (CollectionUtils.isEmpty(list)) {
            unlockTask(task.getWxEnterpriseId());
            return;
        }
        Set<String> midList = list.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
        //根据类型获取状态
        SyncTaskStatusEnum syncTaskStatusEnum = (dataType == PreDealTypeEnum.group.getVal() ? SyncTaskStatusEnum.group_sync
                : (dataType == PreDealTypeEnum.store.getVal() ? SyncTaskStatusEnum.store_sync : SyncTaskStatusEnum.clerk_sync));
        logger.info("处理开始:类型:{}", syncTaskStatusEnum.getVal());
        //处理预处理数据
        preDealService.updateExceptionToPre(taskId);
        dealDepartmentToMq(taskId, midList, syncTaskStatusEnum);
    }


    private String createBaseTask(String wxEnterpriseId, String taskName, String userId, String desc, int taskType) {
        String key = "sync_create_task_" + wxEnterpriseId;
        RedisUtil.lock(key, 10L);
        String taskLock = getTaskLock(wxEnterpriseId);
        if (StringUtils.isNotBlank(taskLock)) {
        	logger.info("【同步锁】taskLock={}",taskLock);
            RedisUtil.unlock(key);
            return null;
        }
        TabHaobanSyncTask tabHaobanSyncTask = new TabHaobanSyncTask();
        tabHaobanSyncTask.setAddUser(userId);
        tabHaobanSyncTask.setTaskName(taskName);
        tabHaobanSyncTask.setTaskDesc(desc);
        tabHaobanSyncTask.setWxEnterpriseId(wxEnterpriseId);
        tabHaobanSyncTask.setStatusFlag(SyncTaskStatusEnum.init.getVal());
        tabHaobanSyncTask.setTaskId(ToolUtil.randomUUID());
        tabHaobanSyncTask.setTaskType(taskType);
        syncTaskService.createTask(tabHaobanSyncTask);
        lockTask(wxEnterpriseId, tabHaobanSyncTask.getTaskId());
        RedisUtil.unlock(key);
        return tabHaobanSyncTask.getTaskId();

    }

    @Override
    public String createTask(String wxEnterpriseId, String taskName, String userId, String desc) {
        return createBaseTask(wxEnterpriseId, taskName, userId, desc, 0);

    }

    @Override
    public String createQywxTask(String wxEnterpriseId, String taskName, String userId, String desc) {
        return createBaseTask(wxEnterpriseId, taskName, userId, desc, SyncTaskTypeEnum.QYWX_CONTACT.getType());

    }


    /**
     * 处理修改数据
     *
     * @param editList
     */
    private void dealEidtSync(List<SyncSingleDealDTO> editList) {
        if (CollectionUtils.isEmpty(editList)) {
            return;
        }
        //先处理修改的
        editList.forEach(syncSingleDealDTO -> {
            // TODO: 2020/4/9  需要优化
            TabHaobanDepartment department = departmentService.selectById(syncSingleDealDTO.getDepartmentId());
            if (!department.getParentDepartmentId().equals(syncSingleDealDTO.getParentDepartmentId())) {
                department.setParentDepartmentId(syncSingleDealDTO.getParentDepartmentId());
                departmentApiService.edit(EntityUtil.changeEntityByJSON(DepartmentDTO.class, department));
            }

        });
    }

    /**
     * 执行新增同步
     *
     * @param taskId
     * @param wxEnterpriseId
     * @param addList
     */
    private void dealAddSync(String taskId, String wxEnterpriseId, List<SyncSingleDealDTO> addList) {

        Map<String, String> pMap = addList.stream().collect(Collectors.toMap(SyncSingleDealDTO::getRelatedId, SyncSingleDealDTO::getParentDepartmentId));
        //获取企业与分组map
        Map<String, List<String>> eidListMap = addList.stream().filter(dto -> {
            String departmentId = dto.getDepartmentId();
            TabHaobanDepartment department = departmentService.selectById(departmentId);
            if (department != null) {
                return false;
            }
            Integer storeFlag = dto.getStoreFlag();
            if (storeFlag.equals(1)) {
                return false;
            }
            return true;
        }).collect(Collectors.groupingBy(SyncSingleDealDTO::getEnterpriseId
                , Collectors.mapping(SyncSingleDealDTO::getRelatedId, Collectors.toList())));

        //放入预处理表
        eidListMap.forEach((k, v) -> {
            //获取所有分组列表
            List<StoreGroupDTO> groupDTOList = storeGroupService.listStoreGroupAndChildren(v.toArray(new String[]{}), k);
            List<String> hasKey=new ArrayList<>();
            List<TabHaobanPreDealLog> dealList = groupDTOList.stream().filter(dto->{
                if (hasKey.contains(dto.getStoreGroupId())) {
                    return false;
                }
                return true;
            }).map(storeGroupDTO -> {
                TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
                dealLog.setDataId(storeGroupDTO.getStoreGroupId());
                if (pMap.containsKey(storeGroupDTO.getStoreGroupId())) {
                    dealLog.setpDataId(pMap.get(storeGroupDTO.getStoreGroupId()));
                } else {
                    dealLog.setpDataId(storeGroupDTO.getParentGroupId());
                }
                dealLog.setDataType(PreDealTypeEnum.group.getVal());
                dealLog.setStatusFlag(0);
                dealLog.setTaskId(taskId);
                dealLog.setEnterpriseId(storeGroupDTO.getEnterpriseId());
                dealLog.setWxEnterpriseId(wxEnterpriseId);
                return dealLog;
            }).collect(Collectors.toList());

            preDealService.insert(dealList);
        });

        //处理门店信息
        List<TabHaobanPreDealLog> storeList = addList.stream().filter(dto -> {
            return dto.getStoreFlag() == 1;
        }).map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dto.getRelatedId());
            dealLog.setpDataId(dto.getParentDepartmentId());
            dealLog.setDataType(1);
            dealLog.setStatusFlag(0);
            dealLog.setEnterpriseId(dto.getEnterpriseId());
            dealLog.setTaskId(taskId);
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());

        preDealService.insert(storeList);
    }

    /**
     * 初始化部门
     *
     * @param taskId
     * @param wxEnterpriseId
     * @param addList
     */
    private void dealDepartmentInit(String taskId, String wxEnterpriseId, List<SyncSingleDealDTO> addList) {
        Set<String> addListMid = addList.stream().filter(dto -> dto.getStoreFlag().equals(0)).map(dto -> dto.getRelatedId()).collect(Collectors.toSet());
        if (CollectionUtils.isNotEmpty(addListMid)) {
            dealDepartmentToMq(taskId, addListMid, SyncTaskStatusEnum.group_sync);
        } else {
            addListMid = addList.stream().filter(dto -> dto.getStoreFlag().equals(1)).map(dto -> dto.getRelatedId()).collect(Collectors.toSet());
            dealDepartmentToMq(taskId, addListMid, SyncTaskStatusEnum.store_sync);
        }
    }

    /**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
//
//        dealList.forEach(relationId -> {
//            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
//            dealParamMqDTO.setTaskId(taskId);
//            dealParamMqDTO.setData(relationId);
//            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
//            GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
//            try {
//                clientInstance.sendMessage("departmentSyncDealMq", JSONObject.toJSONString(dealParamMqDTO));
//            } catch (Exception e) {
//                logger.info("发送失败:{},{}", taskId, relationId);
//                e.printStackTrace();
//            }
//        });

        List<String> listRet = dealList.stream().map(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            return JSONObject.toJSONString(dealParamMqDTO);
        }).collect(Collectors.toList());

        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            clientInstance.sendBatchMessages("departmentSyncDealMq", listRet);
        } catch (Exception e) {
            logger.info("发送失败:{},{}", taskId, JSONObject.toJSONString(listRet));
            e.printStackTrace();
        }
    }


    public String lockTask(String wxEnterpriseId, String taskId) {
        String key = LOCK_KEY + wxEnterpriseId;
        Object cache = RedisUtil.getCache(key);
        if (null == cache) {
            RedisUtil.setCache(key, taskId, 5L, TimeUnit.DAYS);
            return taskId;
        }
        return null;
    }

    @Override
    public boolean closeTask(String wxEnterpriseId) {
        String taskId = getTaskLock(wxEnterpriseId);
        if (taskId != null) {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_close.getVal());
            unlockTask(wxEnterpriseId);
        }
        return true;
    }

    @Override
    public void unlockTask(String wxEnterpriseId) {
        String key = LOCK_KEY + wxEnterpriseId;
        String taskId = (String) RedisUtil.getCache(key);
        if (StringUtils.isNotBlank(taskId)) {
            TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
            if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.compute.getVal())) {
                String sKey = LOCK_SUCCESS_KEY + wxEnterpriseId;
                RedisUtil.setCache(sKey, taskId);
            }
        }
        RedisUtil.delCache(key);
    }

    @Override
    public String getTaskLock(String wxEnterpriseId) {
        String key = LOCK_KEY + wxEnterpriseId;
        return (String) RedisUtil.getCache(key);
    }

    @Override
    public SyncCheckDTO getTaskCheck(String wxEnterpriseId) {
        String taskId = getTaskLock(wxEnterpriseId);
        SyncCheckDTO syncCheckDTO = new SyncCheckDTO();
        String sKey = LOCK_SUCCESS_KEY + wxEnterpriseId;
        if (null == taskId) {
            taskId = (String) RedisUtil.getCache(sKey);
            RedisUtil.delCache(sKey);
        }
        if (null == taskId) {
            return syncCheckDTO;
        }
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        syncCheckDTO.setSyncStatus(syncTask.getStatusFlag());

        PreDealLogInfoDTO dealLogInfoDTO = preDealService.getLastPreDataByTaskId(taskId);
        //一分钟内没执行就重试 并在一天内
        Date date = DateUtils.addMinutes(new Date(), -1);
        Date dayDate = DateUtils.addDays(new Date(), -1);
        if (dealLogInfoDTO.getUpdateTime().before(date)
                &&syncTask.getUpdateTime().before(date)
                &&syncTask.getCreateTime().after(dayDate)) {
            logger.info("重试：{}",taskId);
            syncTaskService.updateTaskStatus(taskId,SyncTaskStatusEnum.exception_close.getVal());
            reDealDepartment(taskId,-1);
            return syncCheckDTO;
        }

        int totalCount = preDealService.countByTaskId(taskId, -1, -1);
        int groupErrCount = preDealService.countByTaskId(taskId, PreDealTypeEnum.group.getVal(), PreDealStatusEnum.exception.getVal());
        int storeErrCount = preDealService.countByTaskId(taskId, PreDealTypeEnum.store.getVal(), PreDealStatusEnum.exception.getVal());
        int clerkErrCount = preDealService.countByTaskId(taskId, PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.exception.getVal());
        int preCount = preDealService.countByTaskId(taskId, -1, PreDealStatusEnum.pre.getVal());


        syncCheckDTO.setTotalDeal(totalCount);
        syncCheckDTO.setGroupErrCount(groupErrCount);
        syncCheckDTO.setStoreErrCount(storeErrCount);
        syncCheckDTO.setStaffErrCount(clerkErrCount);
        syncCheckDTO.setHasDeal(totalCount - preCount);
        return syncCheckDTO;
    }

    @Override
    public Page<SyncTaskDTO> listSyncTask(String wxEnterpriseId, BasePageInfo pageInfo) {
        Page<SyncTaskDTO> page = syncTaskService.listTaskPage(wxEnterpriseId, pageInfo);
        if (CollectionUtils.isEmpty(page.getResult())) {
            return page;
        }
        List<SyncTaskDTO> result = page.getResult();
        for (SyncTaskDTO dto : result) {
            String addUser = dto.getAddUser();
            TabHaobanStaff haobanStaff = staffService.selectById(addUser);
            if (null != haobanStaff) {
                dto.setUserName(haobanStaff.getStaffName());
            } else {
                dto.setUserName("系统管理员");
            }
        }
        return page;
    }

    @Override
    public Page<PreDealLogInfoDTO> listSyncTaskDetail(String wxEnterpriseId, String taskId, BasePageInfo pageInfo) {
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        Page<PreDealLogInfoDTO> page = preDealService.listExceptionByTaskId(taskId, pageInfo);
        if (CollectionUtils.isEmpty(page.getResult())) {
            return page;
        }
        //gic同步微信
        if (syncTask.getTaskType().equals(0)) {
            List<PreDealLogInfoDTO> result = page.getResult();
            for (PreDealLogInfoDTO dto : result) {
                if (dto.getDataType().equals(PreDealTypeEnum.group.getVal())) {
                    StoreGroupDTO midGroup = storeGroupService.getStoreGroupById(dto.getDataId());
                    if (midGroup != null) {
                        dto.setDataName(midGroup.getStoreGroupName());
                        dto.setChainName(midGroup.getStoreGroupNameLevel());
                    }
                } else if (dto.getDataType().equals(PreDealTypeEnum.store.getVal())) {
                    StoreDTO store = storeService.getStore(dto.getDataId());
                    if (null != store) {
                        dto.setDataName(store.getStoreName());
                        dto.setChainName(store.getStoreGroupName());
                        dto.setDataCode(store.getStoreCode());
                    }
                } else if (dto.getDataType().equals(PreDealTypeEnum.clerk.getVal())) {
                    ClerkDTO clerkDTO = clerkService.getClerkByClerkId(dto.getDataId());
                    if (null != clerkDTO) {
                        dto.setDataName(clerkDTO.getClerkName());
                        dto.setChainName(clerkDTO.getStoreName());
                        dto.setDataCode(clerkDTO.getClerkCode());
                    }
                }
            }
        }else if(syncTask.getTaskType().equals(2)){
        	List<PreDealLogInfoDTO> result = page.getResult();
            for (PreDealLogInfoDTO dto : result) {
                dto.setDataCode(dto.getDataId());
                TabHaobanStaff staff = staffService.selectByUserIdAndEnterpriseId(dto.getpDataId(), wxEnterpriseId);
                if(staff != null){
                	dto.setDataName(staff.getStaffName());
                }
//                if(dto.getDataType() == PreDealTypeEnum.friend.getVal()){
//                	dto.setDataName("第三方应用处理");
//                }else if(dto.getDataType() == PreDealTypeEnum.friend_clerk.getVal()){
//                	dto.setDataName("导购处理");
//                }else if(dto.getDataType() == PreDealTypeEnum.friend_merge.getVal()){
//                	dto.setDataName("合并处理");
//                }else if(dto.getDataType() == PreDealTypeEnum.self_friend.getVal()){
//                	dto.setDataName("自建应用处理");
//                }
                //dto.setReason(reason);
            }
        }
        else{
            List<PreDealLogInfoDTO> result = page.getResult();
            for (PreDealLogInfoDTO dto : result) {
                dto.setDataCode(dto.getDataId());
                dto.setDataName(dto.getDataContent());
            }
        }
        return page;
    }


    @Override
    public String dealQywxDepartment(String taskId, String wxEnterpriseId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.HAOBAN_HELP.getVal());
        if (null == secretSetting||secretSetting.getCheckFlag()==0) {
            logger.info("没有配置secret");
            syncTaskService.updateTaskStatus(taskId,SyncTaskStatusEnum.exception_close.getVal());
            unlockTask(wxEnterpriseId);
            return "没有配置secret";
        }
        List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list = qywxDepartmentApiService.listSelfDepartment(wxEnterpriseDTO.getCorpid(), secretSetting.getSecretVal(), 1);
        if (CollectionUtils.isEmpty(list)) {
            logger.info("没有数据同步");
            syncTaskService.updateTaskStatus(taskId,SyncTaskStatusEnum.exception_close.getVal());
            unlockTask(wxEnterpriseId);
            return "没有数据同步或权限没设置全部";
        }
        List<TabHaobanPreDealLog> dealLogList = list.stream().map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dto.getId().toString());
            dealLog.setpDataId(dto.getParentid().toString());
            dealLog.setDataType(0);
            dealLog.setStatusFlag(0);
            dealLog.setTaskId(taskId);
            dealLog.setDataContent(JSONObject.toJSONString(dto));
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());

        preDealService.insert(dealLogList);
        HashSet<String> hashSet = new HashSet<>();
        hashSet.add("1");
        dealDepartmentToMq(taskId, hashSet, SyncTaskStatusEnum.group_sync);
        return null;
    }


    @Override
    public void cleanDiffrence(String wxEnterpriseId, String taskId) {
        List<TabHaobanPreDealLog> groupList = preDealService.listByTaskId(taskId, PreDealTypeEnum.group.getVal(), PreDealStatusEnum.computed.getVal());
        List<TabHaobanPreDealLog> clerkList = preDealService.listByTaskId(taskId, PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.computed.getVal());

        //清空staff
        if (CollectionUtils.isNotEmpty(clerkList)) {
            List<String> clerkIds = clerkList.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toList());
            List<StaffDTO> retLists = staffService.listByUserIdsAndWxEnterpriseId(clerkIds, wxEnterpriseId);
            List<String> staffIds = retLists.stream().map(StaffDTO::getStaffId).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(staffIds)) {
                List<String> delStaffIds = staffService.listDelUserStaffId(wxEnterpriseId, staffIds);
                if (CollectionUtils.isEmpty(delStaffIds)) {
                    logger.info("没有要删除的用户：{}--{}",wxEnterpriseId,taskId);
                }else {
                    staffClerkRelationService.cleanStaffClerk(wxEnterpriseId, delStaffIds);
                    staffService.cleanStaff(wxEnterpriseId, delStaffIds);
                    staffDepartmentRelatedService.cleanStaffDepartment(wxEnterpriseId, delStaffIds);
                }
            }
        }

        //清空cleak
        if (CollectionUtils.isNotEmpty(groupList)) {
            List<String> groupIdList = (List<String>) groupList.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toList());
            departmentService.cleanDepartment(wxEnterpriseId,groupIdList);
        }

        RedisUtil.delLocalCache("department-list-cache-"+wxEnterpriseId);

    }


	@Override
	public String createWxFriendTask(String wxEnterpriseId, String taskName, String staffId, String desc) {
		TabHaobanSyncTask tabHaobanSyncTask = new TabHaobanSyncTask();
        tabHaobanSyncTask.setAddUser(staffId);
        tabHaobanSyncTask.setTaskName(taskName);
        tabHaobanSyncTask.setTaskDesc(desc);
        tabHaobanSyncTask.setWxEnterpriseId(wxEnterpriseId);
        tabHaobanSyncTask.setStatusFlag(SyncTaskStatusEnum.init.getVal());
        tabHaobanSyncTask.setTaskId(ToolUtil.randomUUID());
        tabHaobanSyncTask.setTaskType(2);
        syncTaskService.createTask(tabHaobanSyncTask);
        lockTask(wxEnterpriseId, tabHaobanSyncTask.getTaskId());
        return tabHaobanSyncTask.getTaskId();
	}

	@Override
	public void dealWxFriendClerk(String taskId, String wxEnterpriseId) {
		List<TabHaobanStaffClerkRelation> relationList = staffClerkRelationService.listByWxEnterpriseId(wxEnterpriseId);
		List<TabHaobanClerkMainStoreRelated> mainStoreList = clerkMainStoreRelatedService.listByWxEnterpriseId(wxEnterpriseId);
		Map<String,TabHaobanClerkMainStoreRelated> map = mainStoreList.stream().collect(Collectors.toMap(TabHaobanClerkMainStoreRelated::getStaffId, s -> s));
        List<TabHaobanPreDealLog> dealLogList = relationList.stream().filter(ls->map.get(ls.getStaffId()) != null).map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dto.getWxUserId().toString());
            dealLog.setpDataId(dto.getStaffId().toString());
            dealLog.setDataType(PreDealTypeEnum.friend_clerk.getVal());
            dealLog.setStatusFlag(0);
            dealLog.setTaskId(taskId);
            dealLog.setDataContent(JSONObject.toJSONString(dto));
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());

        preDealService.insert(dealLogList);
        HashSet<String> hashSet = (HashSet<String>) dealLogList.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
        dealFriendToMq(taskId, hashSet, SyncTaskStatusEnum.friend_clerk_sync);
	}

	@Override
    public String createWxFriendTaskSingle(String wxEnterpriseId, String taskName, String staffId, String staffName, int syncTaskType) {
        String lockKey = FRIEND_LOCK + "_lock_" + staffId;
        String staffLockKey = FRIEND_LOCK + staffId;
        RedisUtil.lock(lockKey, 2L);

        Object cache = RedisUtil.getCache(staffLockKey);
        if (null != cache) {
            RedisUtil.unlock(lockKey);
            return null;
        }
        TabHaobanSyncTask tabHaobanSyncTask = new TabHaobanSyncTask();
        tabHaobanSyncTask.setAddUser(staffId);
        tabHaobanSyncTask.setTaskName(SyncTaskTypeEnum.getTaskName(syncTaskType));
        tabHaobanSyncTask.setTaskDesc(staffName);
        tabHaobanSyncTask.setWxEnterpriseId(wxEnterpriseId);
        tabHaobanSyncTask.setStatusFlag(SyncTaskStatusEnum.init.getVal());
        tabHaobanSyncTask.setTaskId(ToolUtil.randomUUID());
        tabHaobanSyncTask.setTaskType(syncTaskType);
        syncTaskService.createTask(tabHaobanSyncTask);
        RedisUtil.setCache(staffLockKey, tabHaobanSyncTask.getTaskId(), 1L, TimeUnit.HOURS);
        RedisUtil.unlock(lockKey);
        return tabHaobanSyncTask.getTaskId();
	}


	@Override
	public void dealWxFriendClerkSingle(String taskId, String wxUserId, String staffId,String wxEnterpriseId) {
		TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
        dealLog.setDataId(wxUserId);
        dealLog.setpDataId(staffId);
        dealLog.setDataType(PreDealTypeEnum.friend_clerk.getVal());
        dealLog.setStatusFlag(0);
        dealLog.setTaskId(taskId);
        dealLog.setDataContent("刷新"+staffId);
        dealLog.setWxEnterpriseId(wxEnterpriseId);
        List<TabHaobanPreDealLog> dealLogList = new ArrayList<TabHaobanPreDealLog>();
        dealLogList.add(dealLog);
        preDealService.insert(dealLogList);
        HashSet<String> hashSet = (HashSet<String>) dealLogList.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
        dealDepartmentToMq(taskId, hashSet, SyncTaskStatusEnum.friend_clerk_sync);
	}

    @Override
    public void dealWxFriendStore(String taskId, String storeId, String wxEnterpriseId) {
        List<StaffClerkRelationDTO> clerkRelationDTOS = staffClerkRelationService.listBindByStoreId(wxEnterpriseId, storeId);
        if (CollectionUtils.isEmpty(clerkRelationDTOS)) {
            logger.info("没有要刷新的：{},{}", wxEnterpriseId, storeId);
            return;
        }
        List<TabHaobanPreDealLog> dealLogList = clerkRelationDTOS.stream().map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dto.getWxUserId());
            dealLog.setpDataId(dto.getStaffId());
            dealLog.setDataType(PreDealTypeEnum.friend_clerk.getVal());
            dealLog.setStatusFlag(0);
            dealLog.setTaskId(taskId);
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());
        preDealService.insert(dealLogList);
        Set<String> hashSet = dealLogList.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
        dealDepartmentToMq(taskId, hashSet, SyncTaskStatusEnum.friend_clerk_sync);
    }

    @Override
    public void quartWxFriendClerk(String res) {
		List<TabHaobanSyncTask> taskList = syncTaskService.listTaskByType(2);
		if(CollectionUtils.isNotEmpty(taskList)){
			for (TabHaobanSyncTask tabHaobanSyncTask : taskList) {
				dealWxFriendClerk(tabHaobanSyncTask.getTaskId(), tabHaobanSyncTask.getWxEnterpriseId());
			}
		}
		
	}
	
	@Override
	public void test(String wxEnterpriseId,String taskId){
		List<TabHaobanStaffClerkRelation> relationList = new ArrayList<TabHaobanStaffClerkRelation>();
		TabHaobanStaffClerkRelation tab = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab1 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab2 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab3 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab4 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab5 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab6 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab7 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab8 = new TabHaobanStaffClerkRelation();
		TabHaobanStaffClerkRelation tab9 = new TabHaobanStaffClerkRelation();
//		tab.setWxUserId("12");
//		tab1.setWxUserId("13989473881");
//		tab2.setWxUserId("1545shiyao");
//		tab3.setWxUserId("15757150876");
//		tab4.setWxUserId("18758285875");
//		tab5.setWxUserId("2224324sdftgret90");
//		tab6.setWxUserId("2341578784");
//		tab7.setWxUserId("234324");
//		tab8.setWxUserId("niweizhong");
		tab9.setWxUserId("QiuWenMin");
		
//		tab.setStaffId("12");
//		tab1.setStaffId("13989473881");
//		tab2.setStaffId("1545shiyao");
//		tab3.setStaffId("15757150876");
//		tab4.setStaffId("18758285875");
//		tab5.setStaffId("2224324sdftgret90");
//		tab6.setStaffId("2341578784");
//		tab7.setStaffId("234324");
//		tab8.setStaffId("niweizhong");
		tab9.setStaffId("QiuWenMin");
//		relationList.add(tab);
//		relationList.add(tab1);
//		relationList.add(tab2);
//		relationList.add(tab3);
//		relationList.add(tab4);
//		relationList.add(tab5);
//		relationList.add(tab6);
//		relationList.add(tab7);
//		relationList.add(tab8);
		relationList.add(tab9);
        List<TabHaobanPreDealLog> dealLogList = relationList.stream().map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dto.getWxUserId().toString());
            dealLog.setpDataId(dto.getStaffId().toString());
            dealLog.setDataType(PreDealTypeEnum.friend_clerk.getVal());
            dealLog.setStatusFlag(0);
            dealLog.setTaskId(taskId);
            dealLog.setDataContent(JSONObject.toJSONString(dto));
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());

        preDealService.insert(dealLogList);
        HashSet<String> hashSet = (HashSet<String>) dealLogList.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
        dealDepartmentToMq(taskId, hashSet, SyncTaskStatusEnum.friend_clerk_sync);
	}
	
	 /**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealFriendToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
//
//        dealList.forEach(relationId -> {
//            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
//            dealParamMqDTO.setTaskId(taskId);
//            dealParamMqDTO.setData(relationId);
//            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
//            GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
//            try {
//                clientInstance.sendMessage("departmentSyncDealMq", JSONObject.toJSONString(dealParamMqDTO));
//            } catch (Exception e) {
//                logger.info("发送失败:{},{}", taskId, relationId);
//                e.printStackTrace();
//            }
//        });

        List<String> listRet = dealList.stream().map(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            return JSONObject.toJSONString(dealParamMqDTO);
        }).collect(Collectors.toList());

        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            clientInstance.sendBatchMessages("friendSyncDealMq", listRet);
        } catch (Exception e) {
            logger.info("发送失败:{},{}", taskId, JSONObject.toJSONString(listRet));
            e.printStackTrace();
        }
    }
}
