package com.gic.haoban.manage.web.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.gic.haoban.auth.api.anno.MenuCheck;
import com.gic.haoban.auth.api.dto.UserRightDetailDTO;
import com.gic.haoban.auth.api.enums.RoleCodeEnum;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.LoginVO;

//import com.gic.redis.data.util.GicRateLimiter;
import com.gic.redis.data.util.GicRateLimiter;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Map;

/**
 * Created 2018/7/20.
 *
 * @author hua
 */
public class WebInterceptor extends HandlerInterceptorAdapter {

    private static Logger logger= LoggerFactory.getLogger(WebInterceptor.class);

//    @Autowired
//    private ClerkRightManageService clerkRightManageService;
    private static String MOUDLE="haoban-manage3-web";

    private void errorResult(HttpServletResponse httpServletResponse, HaoBanErrCode errCode) {
        HaobanResponse response = new HaobanResponse();
        response.setErrorCode(errCode.getCode());
        response.setMessage(errCode.getMsg());
        PrintWriter writer = null;
        try {
            httpServletResponse.setHeader("Content-type", "application/json;charset=UTF-8");
            writer = httpServletResponse.getWriter();
            writer.append(JSONObject.toJSONString(response));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o) throws Exception {
        String requestURI = httpServletRequest.getRequestURI();
        logger.info("post-url:{}", requestURI);
        Map<String, String[]> parameterMap = httpServletRequest.getParameterMap();
        logger.info("post-params:{}", JSONObject.toJSONString(parameterMap));
        if (!(o instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handler = (HandlerMethod) o;
        IgnoreLogin ignoreLogin = handler.getMethodAnnotation(IgnoreLogin.class);
        if (null != ignoreLogin) {
            return true;
        }
        
        Cookie[] cookies = AuthRequestUtil.getRequest().getCookies();
		String token = "";
		if(cookies != null){
			for (Cookie cookie : cookies) {
				if(cookie.getName().equals("token")){
					token = cookie.getValue();
				}
			}
		}
        LoginVO loginUser = (LoginVO) AuthRequestUtil.getAppLoginUser(token);

        if (loginUser == null) {
            this.errorResult(httpServletResponse, HaoBanErrCode.ERR_4);
            logger.info("用户不存在!");
            return false;
        }
        HandlerMethod method = null;
        if (o instanceof HandlerMethod) {
            method = (HandlerMethod) (o);
        }
        if (method == null) {
            return true;
        }
        MenuCheck methodAnnotation = method.getMethodAnnotation(MenuCheck.class);
        if (StringUtils.isNotBlank(loginUser.getWxEnterpriseId())) {
            httpServletRequest.setAttribute("wxEnterpriseId", loginUser.getWxEnterpriseId());
        }
        if (loginUser.getStaffDTO() != null) {
            httpServletRequest.setAttribute("staffDTO", loginUser.getStaffDTO());
        }
        String wxEnterpriseId = loginUser.getWxEnterpriseId();

        GicRateLimiter instance = GicRateLimiter.getInstance(MOUDLE);

        Long acquire = instance.acquire(requestURI, wxEnterpriseId);
        logger.info("限流返回值：{},{}",requestURI,acquire);
        if (acquire==0L) {
            this.errorResult(httpServletResponse, HaoBanErrCode.ERR_13);
            logger.info("限流!");
            return false;
        }
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, ModelAndView modelAndView) throws Exception {
        logger.info("end");
    }

    @Override
    public void afterCompletion(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, Exception e) throws Exception {
        logger.info("end");
    }
}
